==> index.php?download=carddavmiddleware_2.1.2.tar.gz: phonemiddleware/core/README.md <==
<!-- This file is present in both source and website. Please keep in sync -->

<div align="center">
  <img src="assets/images/icon_readme.png" alt="Module icon" width="164">
  <h3>CardDAV Middleware for FreePBX/Incredible PBX/izpbx</h3>
</div>

## About
CardDAV Middleware (formerly PhoneMiddleware) is a module for FreePBX/Incredible PBX/izpbx and similar systems that can read a cardDAV server and return inbound and Outbound CNAM (read: caller ID/name) and a full XML phonebook that your phone (or any other device) can read!

## Installation and Usage
1. The Module depends on <a target="_blank" href="https://github.com/POSSA/freepbx-Outbound_CNAM">Outbound CNAM</a>. This module is officially available through the "Module Admin" interface by enabling the "Unsupported" repository. You can then "Check Online" and install it directly
2. Upload <a href="https://carddavmiddleware.massi-x.dev">CardDAV Middleware</a> through "Module Admin" and install it
3. Go to Applications > CardDAV Middleware and fill everything with your data
4. Click "Auto Configure" to automagically setup your system for Inbound and Outbound CNAM! (If you have any trouble with Auto Configure you will find all the information to configure your system manually right inside that popup)

<details>
<summary>Note for FPBX 15 users</summary>
<div class="content">
Although the module is compatible with v.15 systems, you must keep in mind the following:
<ul>
<li>You might need to update PHP manually to a version above 7.2.5</li>
<li>You must install <a target="_blank" href="http://pbxossa.org/files/outcnam/">Outbound CNAM</a> manually by uploading it through "Module Admin" because on these systems is not (at least yet) available in the official repo</li>
</ul>
</div>
</details>

_**Where to find the XML phonebook URL:**_ Open the module interface, you will find a link right at the bottom that you can copy and use on any of your devices!<br>
_**A note about security:**_ The module does not implement any type of security (like HTTPS or TLS) so please, use it only on internal/trusted networks or behind a Proxy/VPN

<h2 id="changelog">Changelog</h2>

<b id="2.1.2">2.1.2</b>
<ul>
<li>Updated <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware-ui">UI</a> to version 1.2</li>
<li>Fixed inability to install on v.15 systems</li>
<li>Updated module icon</li>
</ul>

<details>
<summary><span style="cursor: pointer">Older versions</span></summary>
<div class="content">
<b id="2.1.1">2.1.1</b>
<ul>
<li>Updated <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware-ui">UI</a> to version 1.1.1</li>
<li>Fixed SSL errors in superfecta</li>
<li>Use centralized common mail "FROM" field (provided by framework)</li>
<li>Plus all the great improvements in <a href="#2.1.0">2.1.0</a>!</li>
</ul>
<b id="2.1.0">2.1.0</b>
<ul>
<li>Updated <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware-ui">UI</a> to version 1.1.0</li>
<li>Officially tested and compatible with <b>Incredible PBX</b> and <b>izpbx</b>!</li>
<li>Improved logging system to include details for errors that happens in background</li>
<li>Fixed error "could not interpret numbers after plus sign" and improved handling of malformed numbers</li>
<li>Fixed inability to load data if cache was set to 0 (no cache)</li>
<li>Correctly handle toll-free numbers in XML Phonebook (these will not have the national prefix added)</li>
<li>Automatically extract your country code on installation and apply it (needs an internet connection)</li>
<li>Allows for custom GET requests to have multiple XML Phonebooks for different phone types (implementation of new UI method). You can now include the phone type inside the request in the format "?type=[TYPE]" to load the appropriate phonebook</li>
<li>Allows for SSL bypass (implementation of new UI method) to load from you carddav server even if the certificate is not valid (not recommended!)</li>
<li>Fix for donation notifications sent multiple times</li>
<li>Updated dependencies and website URLs</li>
</ul>
<b id="2.0.0">2.0.0</b>
<ul>
<li>New revolutionary version! Fixes and improvements everywhere, be careful that if you were using previous version you will need to reconfigure your system because of the many ground-breaking changes (this is now so easy thanks to auto configure!)</li>
<li>Let's start with compatibily notes: This version requires at least PHP 7.2.5, so if you are on FreePBX 15 you will likely need to update it manually (using `pkg updgrade` from shell). You will likely need to update dependent modules too, for this is enough to run an update from the "Module Admin" interface</li>
<li>Entire UI revamp! All the interface is now better organized and clean, with great explanation for every feature. All the inputs are now easy to manage with guided types and values to reduce the possibility of misconfigurations</li>
<li>New simple yet advanced CardDAV setup! You don't need to input exact URLs anymore, with the help of the superb <a target="_blank" href="https://github.com/mstilkerich/carddavclient">Mstilkerich CardDAV Client</a> library they will be resolved automatically!</li>
<li>New multiple addressbook! You can now have multiple addressbooks (closes issue <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/2">#2</a>) and even reorder them!</li>
<li>New "Auto Configure" option! This magic button allows with a simple click to configure your entire FreePBX system for incoming and outcoming CNAM, no more hard manual configure needed!</li>
<li>New notification section! For any error or info a new section has been added at the top of the UI. Click the bell to find all the notifications waiting for you!</li>
<li>New Superfecta Compatibility option! Toggling compatibility to "Full" allows Superfecta to correctly work with multiple schemes (so you can have CardDAV Middleware followed by Who Called for example). "Standard" compatibility is useful if you don't mind about multiple schemes and instead you like to have reports of error in the library directly on your phone screen. Requires Superfecta version 15.0.4/16.0.26</li>
<li>New SPAM option! By adding a category (or tag or label, name depends on the client you are using) named "SPAM" inside the interested vCards you can flag a contact as SPAM. Requires Superfecta version 15.0.4/16.0.26</li>
<li>New notification via mail! You can choose to receive an email when you get a notification, so in case of an error you will be instantly notified!</li>
<li>Now the module tries to automatically extract the country code from the number. Only if this is missing it will fallback to the one provided (that should be the one of the country your system is)</li>
<li>Fixed some issues with SNOM devices (see <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/11">#11</a>)</li>
<li>Fixed some issues in the phonebook</li>
<li>Fixed some issues in CNAM matching and switched to POST based queries to get even the "+" sign if present in the query. Requires Superfecta version 15.0.4/16.0.26</li>
<li>Removed "sub" options in output construct, they were too hard to maintain and not that useful. You can now choose from a pre-defined list of options</li>
<li>Update are now delivered through standard FreePBX methods. That's it, you will find the module listed with all the other one. Remember to enable update notifications in the "Module Admin" section</li>
<li>Implemented search with FreePBX search bar</li>
<li>Closed issue <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/10">#10</a></li>
</ul>

<b id="1.2.0">1.2.0</b>
<ul>
<li>Module officially signed with FreePBX key! Now you are sure this comes from me and is not tampered (ported to old versions too)</li>
<li>Completed <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/4">#4</a></li>
<li>Fixed Uninstall</li>
<li>Various improvements and UI fixes</li>
</ul>

<b id="1.1.0">1.1.0</b>
<ul>
<li>New update notifications via e-mail so you won't miss an update anymore!</li>
<li>Refactoring of the code</li>
</ul>

<b id="1.0.7">1.0.7</b>
<ul>
<li>Completed <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/8">#8</a></li>
<li>Dramatically improved CNAM recognition by switching to regex (up to 10x faster!)</li>
</ul>

<b id="1.0.6">1.0.6</b>
<ul>
<li>Implemented <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/4">#4</a>: Custom display names for incoming calls. Now you can define custom parameters as better described in the interface. For example, you can get the name followed by the nickname or the organization name or the title, reverse them… Powerful! (Still needs to be completed for the xml phonebook)</li>
<li>Fixed <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/7">#7</a>: Correctly replacing + with 00</li>
<li>Fixed <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/6">#6</a>: Exactly three elements even if empty for Snom phones</li>
<li>Fixed <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/5">#5</a>: ‘Office’ tag instead of ‘Other’ for Snom phones</li>
<li>Now module folder url is case insensitive</li>
<li>Refactoring of various components and better explanations in the interface</li>
</ul>

<b id="1.0.5">1.0.5</b>
<ul>
<li>Fixed issue <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/3">#3</a></li>
</ul>

<b id="1.0.4">1.0.4</b>
<ul>
<li>Better handling of connection fails on phone screen</li>
<li>Better update handling and config restore</li>
<li>Various bugfix</li>
</ul>

<b id="1.0.3">1.0.3 -REMOVED-</b>
<ul>
<li>Fixed preference lost after update</li>
<li>Added license in the module page</li>
<li>Added donation links</li>
</ul>

<b id="1.0.2">1.0.2 -REMOVED-</b>
<ul>
<li>Fixed license texts</li>
</ul>

<b id="1.0.1">1.0.1 -REMOVED-</b>
<ul>
<li>Whoa! First module version. Officially compatible with FreePBX distro 15 and up.</li>
</ul>
</div>
</details>

## License
`SPDX-License-Identifier: Proprietary`<br>
This work is licensed under a <a target="_blank" href="https://carddavmiddleware.massi-x.dev/LICENSE">proprietary license</a>.<br>
By using this module you agree to the terms. Licenses for the included libraries are available below, credits go to the original author only.
- [Composer](https://github.com/composer/composer)
- [Giggsey libphonenumber for PHP](https://github.com/giggsey/libphonenumber-for-php)
- [Giggsey Locale](https://github.com/giggsey/Locale)
- [Guzzle](https://github.com/guzzle/guzzle)
- [Guzzle Promises](https://github.com/guzzle/promises)
- [Guzzle Psr7](https://github.com/guzzle/psr7)
- [Mstilkerich CardDAV Client](https://github.com/mstilkerich/carddavclient)
- [PHP-FIG HTTP Client](https://www.php-fig.org/psr/psr-18)
- [PHP-FIG HTTP Factory](https://www.php-fig.org/psr/psr-17)
- [PHP-FIG HTTP Message](https://www.php-fig.org/psr/psr-7)
- [PHP-FIG Logger](https://www.php-fig.org/psr/psr-3)
- [Ralouphie getallheaders](https://github.com/ralouphie/getallheaders)
- [Sabre URI](https://github.com/sabre-io/uri)
- [Sabre vobject](https://github.com/sabre-io/vobject)
- [Sabre XML](https://github.com/sabre-io/xml)
- [Symfony Deprecation Contracts](https://github.com/symfony/deprecation-contracts)
- [Symfony Polyfill / Mbstring](https://github.com/symfony/polyfill-mbstring)

==> index.php?download=carddavmiddleware_2.3.1.tar.gz: phonemiddleware/core/README.md <==
<!-- This file is present in both source and website. Please keep in sync -->

<div align="center">
  <img src="assets/images/icon_readme.png" alt="Module icon" width="164">
  <h3>CardDAV Middleware for FreePBX/Incredible PBX/izpbx</h3>
</div>

## About
CardDAV Middleware (formerly PhoneMiddleware) is a module for FreePBX/Incredible PBX/izpbx and similar systems that can read a cardDAV server and return inbound and Outbound CNAM (read: caller ID/name) and a full XML phonebook that your phone (or any other device) can read!

## Installation and Usage
1. The Module depends on <a target="_blank" href="https://github.com/POSSA/freepbx-Outbound_CNAM">Outbound CNAM</a>. This module is officially available through the "Module Admin" interface by enabling the "Unsupported" repository (in the top row). You can then "Check Online" and install it directly
2. Return to the "Module Admin" interface and choose "Upload modules", make sure you select the Download option from the dropdown then copy and paste the URL <a href="https://carddavmiddleware.massi-x.dev/carddavmiddleware_latest.tar.gz">https://carddavmiddleware.massi-x.dev/carddavmiddleware_latest.tar.gz</a> in the input field, ultimately click "Download (From Web)" to Download the module to your system
3. Return to the "Module Admin" interface again and install the just downloaded module (search for CardDAV Middleware inside the table)
4. Go to Applications > CardDAV Middleware and fill the inputs with your data
5. Click "Auto Configure" to automagically setup your system for Inbound and Outbound CNAM! (If you have any trouble with Auto Configure you will find all the information to configure your system manually right inside that popup)

<details>
<summary>Note for FPBX 15 users</summary>
<div class="content">
Although the module is compatible with v.15 systems, you must keep in mind the following:
<ul>
<li>You might need to update PHP manually to a version above 7.2.5</li>
<li>You must install <a target="_blank" href="http://pbxossa.org/files/outcnam/">Outbound CNAM</a> manually by uploading it through "Module Admin" because on these old systems is not available in the repo</li>
</ul>
</div>
</details>

_**Where to find the XML phonebook URL:**_ Open the module interface, you will find a link right at the bottom that you can copy and use on any of your devices!<br>
_**A note about security:**_ The module is not intended to be bulletproof against malicious attacks. You can enhance the security by enabling the password protection for the XML Phonebook and enabling (and forcing) SSL globally on your PBX.<br>
_**Where could I get support for the module?**_ If you have any problem with the module you can reach to me over <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware-ui/issues/">here</a>, but please first check for open issues!

<h2 id="changelog">Changelog</h2>

<b id="2.3.1">2.3.1</b>
<ul>
<li>Updated UI to version 1.4.1 (see <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware-ui/releases/tag/v1.4.1">here</a> for the full changelog - Warning! Contains some juicy features and fixes!)</li>
<li>Added ability to choose between first/middle/last name in output construct</li>
<li>Added ability to add custom separators between parameters in output construct (closes <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/18">#18</a>)</li>
<li>Even faster cnam response thanks to a new dedicated cache!</li>
<li>Parameters for carddavtoxml.php are now case-insensitive</li>
<li>Fix an issue that could lead to failure if the cache was set to '0'</li>
<li>Automatically merge similar notifications to reduce the number of insignificant ones</li>
</ul>

<details>
<summary><span style="cursor: pointer">Older versions</span></summary>
<div class="content">
<b id="2.2.0">2.2.0</b>
<ul>
<li>Updated <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware-ui">UI</a> to version 1.3</li>
<li>Officially compatible with PBX version 17+</li>
<li>Implemented new "PASSTHROUGH" type for xml phonebook - this leaves labels intact in the output so that you can parse and use them anywhere!</li>
<li>You can now request a basic authentication when accessing the XML phonebook (optional)</li>
<li>Improved stylesheets for better compatibility vith PBX 17+</li>
<li>Add tutorial button</li>
</ul>
<b id="2.1.2">2.1.2</b>
<ul>
<li>Updated <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware-ui">UI</a> to version 1.2</li>
<li>Fixed inability to install on v.15 systems</li>
<li>Updated module icon</li>
</ul>
<b id="2.1.1">2.1.1</b>
<ul>
<li>Updated <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware-ui">UI</a> to version 1.1.1</li>
<li>Fixed SSL errors in superfecta</li>
<li>Use centralized common mail "FROM" field (provided by framework)</li>
<li>Plus all the great improvements in <a href="#2.1.0">2.1.0</a>!</li>
</ul>
<b id="2.1.0">2.1.0</b>
<ul>
<li>Updated <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware-ui">UI</a> to version 1.1.0</li>
<li>Officially tested and compatible with <b>Incredible PBX</b> and <b>izpbx</b>!</li>
<li>Improved logging system to include details for errors that happens in background</li>
<li>Fixed error "could not interpret numbers after plus sign" and improved handling of malformed numbers</li>
<li>Fixed inability to load data if cache was set to 0 (no cache)</li>
<li>Correctly handle toll-free numbers in XML Phonebook (these will not have the national prefix added)</li>
<li>Automatically extract your country code on installation and apply it (needs an internet connection)</li>
<li>Allows for custom GET requests to have multiple XML Phonebooks for different phone types (implementation of new UI method). You can now include the phone type inside the request in the format "?type=[TYPE]" to load the appropriate phonebook</li>
<li>Allows for SSL bypass (implementation of new UI method) to load from you carddav server even if the certificate is not valid (not recommended!)</li>
<li>Fix for donation notifications sent multiple times</li>
<li>Updated dependencies and website URLs</li>
</ul>
<b id="2.0.0">2.0.0</b>
<ul>
<li>New revolutionary version! Fixes and improvements everywhere, be careful that if you were using previous version you will need to reconfigure your system because of the many ground-breaking changes (this is now so easy thanks to auto configure!)</li>
<li>Let's start with compatibily notes: This version requires at least PHP 7.2.5, so if you are on FreePBX 15 you will likely need to update it manually (using `pkg updgrade` from shell). You will likely need to update dependent modules too, for this is enough to run an update from the "Module Admin" interface</li>
<li>Entire UI revamp! All the interface is now better organized and clean, with great explanation for every feature. All the inputs are now easy to manage with guided types and values to reduce the possibility of misconfigurations</li>
<li>New simple yet advanced CardDAV setup! You don't need to input exact URLs anymore, with the help of the superb <a target="_blank" href="https://github.com/mstilkerich/carddavclient">Mstilkerich CardDAV Client</a> library they will be resolved automatically!</li>
<li>New multiple addressbook! You can now have multiple addressbooks (closes issue <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/2">#2</a>) and even reorder them!</li>
<li>New "Auto Configure" option! This magic button allows with a simple click to configure your entire FreePBX system for incoming and outcoming CNAM, no more hard manual configure needed!</li>
<li>New notification section! For any error or info a new section has been added at the top of the UI. Click the bell to find all the notifications waiting for you!</li>
<li>New Superfecta Compatibility option! Toggling compatibility to "Full" allows Superfecta to correctly work with multiple schemes (so you can have CardDAV Middleware followed by Who Called for example). "Standard" compatibility is useful if you don't mind about multiple schemes and instead you like to have reports of error in the library directly on your phone screen. Requires Superfecta version 15.0.4/16.0.26</li>
<li>New SPAM option! By adding a category (or tag or label, name depends on the client you are using) named "SPAM" inside the interested vCards you can flag a contact as SPAM. Requires Superfecta version 15.0.4/16.0.26</li>
<li>New notification via mail! You can choose to receive an email when you get a notification, so in case of an error you will be instantly notified!</li>
<li>Now the module tries to automatically extract the country code from the number. Only if this is missing it will fallback to the one provided (that should be the one of the country your system is)</li>
<li>Fixed some issues with SNOM devices (see <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/11">#11</a>)</li>
<li>Fixed some issues in the phonebook</li>
<li>Fixed some issues in CNAM matching and switched to POST based queries to get even the "+" sign if present in the query. Requires Superfecta version 15.0.4/16.0.26</li>
<li>Removed "sub" options in output construct, they were too hard to maintain and not that useful. You can now choose from a pre-defined list of options</li>
<li>Update are now delivered through standard FreePBX methods. That's it, you will find the module listed with all the other one. Remember to enable update notifications in the "Module Admin" section</li>
<li>Implemented search with FreePBX search bar</li>
<li>Closed issue <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/10">#10</a></li>
</ul>

<b id="1.2.0">1.2.0</b>
<ul>
<li>Module officially signed with FreePBX key! Now you are sure this comes from me and is not tampered (ported to old versions too)</li>
<li>Completed <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/4">#4</a></li>
<li>Fixed Uninstall</li>
<li>Various improvements and UI fixes</li>
</ul>

<b id="1.1.0">1.1.0</b>
<ul>
<li>New update notifications via e-mail so you won't miss an update anymore!</li>
<li>Refactoring of the code</li>
</ul>

<b id="1.0.7">1.0.7</b>
<ul>
<li>Completed <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/8">#8</a></li>
<li>Dramatically improved CNAM recognition by switching to regex (up to 10x faster!)</li>
</ul>

<b id="1.0.6">1.0.6</b>
<ul>
<li>Implemented <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/4">#4</a>: Custom display names for incoming calls. Now you can define custom parameters as better described in the interface. For example, you can get the name followed by the nickname or the organization name or the title, reverse them… Powerful! (Still needs to be completed for the xml phonebook)</li>
<li>Fixed <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/7">#7</a>: Correctly replacing + with 00</li>
<li>Fixed <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/6">#6</a>: Exactly three elements even if empty for Snom phones</li>
<li>Fixed <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/5">#5</a>: ‘Office’ tag instead of ‘Other’ for Snom phones</li>
<li>Now module folder url is case insensitive</li>
<li>Refactoring of various components and better explanations in the interface</li>
</ul>

<b id="1.0.5">1.0.5</b>
<ul>
<li>Fixed issue <a target="_blank" href="https://github.com/Massi-X/freepbx-carddavmiddleware/issues/3">#3</a></li>
</ul>

<b id="1.0.4">1.0.4</b>
<ul>
<li>Better handling of connection fails on phone screen</li>
<li>Better update handling and config restore</li>
<li>Various bugfix</li>
</ul>

<b id="1.0.3">1.0.3 -REMOVED-</b>
<ul>
<li>Fixed preference lost after update</li>
<li>Added license in the module page</li>
<li>Added donation links</li>
</ul>

<b id="1.0.2">1.0.2 -REMOVED-</b>
<ul>
<li>Fixed license texts</li>
</ul>

<b id="1.0.1">1.0.1 -REMOVED-</b>
<ul>
<li>Whoa! First module version. Officially compatible with FreePBX distro 15 and up.</li>
</ul>
</div>
</details>

## License
`SPDX-License-Identifier: Proprietary`<br>
This work is licensed under a <a target="_blank" href="https://carddavmiddleware.massi-x.dev/LICENSE">proprietary license</a>.<br>
By using this module you agree to the terms. Licenses for the included libraries are available below, credits go to the original author only.
- [Composer](https://github.com/composer/composer)
- [Giggsey libphonenumber for PHP](https://github.com/giggsey/libphonenumber-for-php)
- [Giggsey Locale](https://github.com/giggsey/Locale)
- [Guzzle](https://github.com/guzzle/guzzle)
- [Guzzle Promises](https://github.com/guzzle/promises)
- [Guzzle Psr7](https://github.com/guzzle/psr7)
- [Mstilkerich CardDAV Client](https://github.com/mstilkerich/carddavclient)
- [PHP-FIG HTTP Client](https://www.php-fig.org/psr/psr-18)
- [PHP-FIG HTTP Factory](https://www.php-fig.org/psr/psr-17)
- [PHP-FIG HTTP Message](https://www.php-fig.org/psr/psr-7)
- [PHP-FIG Logger](https://www.php-fig.org/psr/psr-3)
- [Ralouphie getallheaders](https://github.com/ralouphie/getallheaders)
- [Sabre URI](https://github.com/sabre-io/uri)
- [Sabre vobject](https://github.com/sabre-io/vobject)
- [Sabre XML](https://github.com/sabre-io/xml)
- [Symfony Deprecation Contracts](https://github.com/symfony/deprecation-contracts)
- [Symfony Polyfill / Mbstring](https://github.com/symfony/polyfill-mbstring)
